#!/usr/bin/env python
# encoding: utf-8

from __future__ import (absolute_import, division, print_function)
import os
import sys
import logging
import subprocess as sp
import starseqr_utils as su

logger = logging.getLogger("STAR-SEQR")


def run_star(fq1, fq2, args):
    '''
    Run STAR alignment for RNA or DNA with different sensitivity parameters.
    '''
    logger.info("Starting STAR Alignment")
    if not os.path.isfile(args.prefix + ".Chimeric.out.junction"):
        if args.nucleic_type == "DNA":
            if not os.path.isdir(args.star_index):
                logger.error("Error: STAR index was not found at " + args.star_index + " Please update the path.", exc_info=True)
                sys.exit(1)
            STAR_args = ['STAR', '--readFilesIn', fq1, fq2, '--readFilesCommand', 'zcat',
                         '--runThreadN', args.threads, '--genomeDir', args.star_index,
                         '--outFileNamePrefix ', args.prefix + ".", '--outSAMtype', 'None',
                         '--alignIntronMax', 1, '--alignMatesGapMax', 0, '--sjdbOverhang', 0,
                         '--chimOutType', 'SeparateSAMold', '--chimScoreJunctionNonGTAG', 0]
            # choose sensitivity mode
            if (args.mode == 0):
                sens_params = ['--chimSegmentMin', 10, '--chimJunctionOverhangMin', 10,
                               '--chimScoreMin', 1, '--chimScoreDropMax', 20,
                               '--chimScoreSeparation', 10, '--chimSegmentReadGapMax', 0]
            elif (args.mode == 1):
                sens_params = ['--chimSegmentMin', 7, '--chimJunctionOverhangMin', 7,
                               '--chimScoreMin', 1, '--chimScoreDropMax', 20,
                               '--chimScoreSeparation', 5, '--chimSegmentReadGapMax', 0]
            STAR_args.extend(sens_params)
            # Need to convert all to string
            STAR_args = list(map(str, STAR_args))
        elif args.nucleic_type == "RNA":
            if not os.path.isdir(args.star_index):
                logger.error("Error: STAR index was not found at " + args.star_index + " Please update the path.", exc_info=True)
                sys.exit(1)
            # outFilterMultimapNmax does not have any effect on chimeric alignments
            # chimSegmentReadGapMax allows for variation at breakpoints
            # chimScoreDropMax is the difference in aligned read length and total read length due to clipping
            # chimMainSegmentMultNmax - new parameter >2.5b.
            # mismatches at sj: (1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3) GC/AG and CT/GC motif, (4) AT/AC and GT/AT motif.
            STAR_args = ['STAR', '--readFilesIn', fq1, fq2, '--readFilesCommand', 'zcat',
                         '--runThreadN', str(args.threads), '--genomeDir', args.star_index,
                         '--outFileNamePrefix ', args.prefix + ".", '--outSAMtype', 'None',
                         '--chimOutType', 'SeparateSAMold', '--chimScoreJunctionNonGTAG', -1,
                         '--alignSJDBoverhangMin', 3, '--outFilterMultimapScoreRange', 1,
                         '--outFilterMultimapNmax', 5]
            # choose sensitivity mode
            if (args.mode == 0):
                sens_params = ['--chimSegmentMin', 12, '--chimJunctionOverhangMin', 12,
                               '--chimScoreMin', 1, '--chimScoreDropMax', 20,
                               '--chimScoreSeparation', 10, '--chimSegmentReadGapMax', 3,
                               '--chimFilter', 'None', '--twopassMode', "Basic",
                               '--alignSJstitchMismatchNmax', 5, -1, 5, 5]  # ,
                               # '--chimMainSegmentMultNmax', 1]
            elif (args.mode == 1):
                sens_params = ['--chimSegmentMin', 10, '--chimJunctionOverhangMin', 10,
                               '--chimScoreMin', 0, '--chimScoreDropMax', 30,
                               '--chimScoreSeparation', 10, '--chimSegmentReadGapMax', 3,
                               '--chimFilter', 'None', '--twopassMode', "Basic",
                               '--alignSJstitchMismatchNmax', 5, -1, 5, 5]  # ,
                               # '--chimMainSegmentMultNmax', 1]
            STAR_args.extend(sens_params)
            # Need to convert all to string
            STAR_args = list(map(str, STAR_args))
        else:
            logger.error("Need to define nucleic_type as either RNA or DNA")
            sys.exit(1)
        logger.info("*STAR Command: " + " ".join(STAR_args))
        # run STAR
        try:
            p = sp.Popen(STAR_args, stdout=sp.PIPE, stderr=sp.PIPE)
            stdout, stderr = p.communicate()
            if stdout:
                logger.info(stdout)
            if stderr:
                logger.error(stderr)
            if p.returncode != 0:
                logger.error("Error: STAR failed", exc_info=True)
                sys.exit(1)
        except (OSError) as o:
            logger.error("Exception: " + str(o))
            logger.error("STAR Failed", exc_info=True)
            sys.exit(1)
        su.common.check_file_exists(args.prefix + ".Chimeric.out.junction")
        su.common.check_file_exists(args.prefix + ".Chimeric.out.sam")
    else:
        su.common.check_file_exists(args.prefix + ".Chimeric.out.junction")
        su.common.check_file_exists(args.prefix + ".Chimeric.out.sam")
        logger.warn("Skipping STAR alignment as files already exist!")
    logger.info("STAR Alignment Finished!")



#!/usr/bin/env python

from setuptools import setup, find_packages, Extension
from io import open
from sys import stderr


def get_version(string):
    """ Parse the version number variable __version__ from a script. """
    import re
    version_re = r"^__version__ = ['\"]([^'\"]*)['\"]"
    version_str = re.search(version_re, string, re.M).group(1)
    return version_str


libssw_ext = Extension('_libssw_ext', sources=['ssw/src/ssw.c'], include_dirs=['ssw/src/'])

setup(
    name='starseqr',
    version=get_version(open('starseqr_utils/__init__.py', encoding='utf-8').read()),
    description='RNA-Fusion Calling with STAR',
    long_description=open('README.rst', encoding='utf-8').read(),
    license='Custom',
    author='Jeff Jasper',
    author_email='jasper1918@gmail.com',
    url='https://github.com/ExpressionAnalysis/STAR-SEQR',
    packages=find_packages(),
    install_requires=['six', 'pandas>=0.18.0', 'pysam>=0.9.0', 'primer3-py', 'intervaltree_bio'],
    ext_modules=[libssw_ext],
    package_data={"starseqr_utils": ["resources/*"]},
    scripts=['starseqr.py'],
    zip_safe=False,
    test_suite='nose.collector',
    tests_require=['nose'],
    keywords=['rna', 'rna-seq', 'fusions', 'chimeric', 'star']
)

stderr.write("Installation was successful!\n")



import sys
import argparse
from svtools.external_cmd import ExternalCmd

class GenotypeVariants(ExternalCmd):
    def __init__(self):
        super(GenotypeVariants, self).__init__('svtyper', 'bin/svtyper/svtyper')

    @staticmethod
    def svtyper_option_lut():
        opts = { 
                'bam' : '-B',
                'split_bam' : '-S',
                'input_vcf' : '-i',
                'output_vcf' : '-o',
                'splflank' : '-f',
                'discflank' : '-F',
                'split_weight' : '--split_weight',
                'disc_weight' : '--disc_weight',
                'num_samp' : '-n',
                'legacy' : '-M',
                'debug' : '--debug',
                }
        return opts

def description():
    return 'compute genotype of structural variants based on breakpoint depth'

def add_arguments_to_parser(parser):
    parser.add_argument('-B', '--bam', type=str, required=True, help='BAM file(s), comma-separated if genotyping multiple BAMs')
    parser.add_argument('-S', '--split_bam', type=str, required=False, help='split-read bam file for sample, comma-separated if genotyping multiple BAMs')
    parser.add_argument('-i', '--input_vcf', help='VCF input (default: stdin)')
    parser.add_argument('-o', '--output_vcf', help='output VCF to write (default: stdout)')
    parser.add_argument('-f', '--splflank', type=int, required=False, default=20, help='min number of split read query bases flanking breakpoint on either side [20]')
    parser.add_argument('-F', '--discflank', type=int, required=False, default=20, help='min number of discordant read query bases flanking breakpoint on either side. (should not exceed read length) [20]')
    parser.add_argument('--split_weight', type=float, required=False, default=1, help='weight for split reads [1]')
    parser.add_argument('--disc_weight', type=float, required=False, default=1, help='weight for discordant paired-end reads [1]')
    parser.add_argument('-n', dest='num_samp', type=int, required=False, default=1000000, help='number of pairs to sample from BAM file for building insert size distribution [1000000]')
    parser.add_argument('-M', action='store_true', dest='legacy', required=False, help='split reads are flagged as secondary, not supplementary. For compatibility with legacy BWA-MEM "-M" flag')
    parser.add_argument('--debug', action='store_true', help='debugging verbosity')
    parser.set_defaults(entry_point=run_from_args)

def command_parser():
    parser = argparse.ArgumentParser(description=description())
    add_arguments_to_parser(parser)
    return parser

def run_from_args(args):
    argdict = vars(args)

    genotyper = GenotypeVariants()
    opts = list()
    optlut = genotyper.svtyper_option_lut()
    for variable, value in argdict.iteritems():
        if variable != 'entry_point' and value not in (False, None):
            opts.extend([optlut[variable], str(value)])
    genotyper.run_cmd_with_options(opts)

if __name__ == "__main__":
    parser = command_parser()
    args = parser.parse_args()
    sys.exit(args.entry_point(args))



import sys

class Genotype(object):
    '''
    This class stores information about each sample.
    '''
    def __init__(self, variant, value_list):
        '''
        Initialize the class. All instances have a GT field, 
        but that is enforced in the Variant class.
        '''
        self.value_list = value_list
        self.variant = variant

    def __eq__(self, other):
        return self.get_gt_string() == other.get_gt_string()
    
    def set_format(self, field, value):
        '''
        Set information for an individual format field.
        '''
        if field in self.variant.format_set:
            if field in self.variant.format_dict:
                i = self.variant.format_dict[field]
                self._set_value(i, value)
            else:
                num_fields = len(self.variant.format_dict)
                self.variant.format_dict[field] = num_fields
                self._set_value(num_fields, value)
        else:
            sys.stderr.write('\nError: invalid FORMAT field, \"' + field + '\"\n')
            sys.exit(1)

    def _set_value(self, index, value):
        try:
            self.value_list[index] = value
        except IndexError:
            self.value_list.extend(['.'] * (index - len(self.value_list)))
            self.value_list.append(value)

    def get_format(self, field):
        '''
        Get value of particular field key
        '''
        try:
            return self.value_list[self.variant.format_dict[field]]
        except IndexError:
            if field != 'GT':
                return '.'
            else:
                return './.'

    def get_gt_string(self):
        '''
        Convert object back to string. 
        
        If some values are missing (at the end for example) they are printed out as 
        all format fields present in any Genotype instance in the Variant line
        are tracked.
        '''
        g_list = list()
        for f in self.variant.format_list:
            if f.id in self.variant.format_dict:
                value = self.get_format(f.id)
                if type(value) == float:
                    g_list.append('%0.2f' % value)
                else:
                    g_list.append(str(value))
        return ':'.join(g_list)




import sys
import argparse
from svtools.bedpe import Bedpe
import svtools.utils as su

class BedpetoBlockedBedConverter(object):
    '''
    Class to convert Bedpe lines to BlockedBed (Bed12)
    '''
    def __init__(self, name, max_dist):
        '''
        Initialize parameters
        '''
        self.max_dist = max_dist
        self.name = name
        self.coordinate_buffer = 500
        self.distant_color = '204,204,204' #gray
        self.unknown_close_color = '128,0,128'
        self.color_table = { 
                'DEL': '153,0,0', #red
                'DUP': '0,102,0', #green
                'INV': '0,51,204', #blue
                'BND': self.distant_color
                }
    
    def track_name(self):
        '''
        Return the track name for the output bedpe. Assumes name is a valid string
        '''
        return "track name=" + self.name + " itemRgb=On\n"

    def get_color(self, svtype, span):
        '''
        Get the color for the Bed12 entries
        '''
        if span <= self.max_dist:
            return self.color_table.get(svtype, self.unknown_close_color)
        else:
            return self.distant_color

    @staticmethod
    def bed12_name(svtype, bedpe_name, allele_frequency, strands=None):
        '''
        Construct the name for the Bed12 entry.
        '''
        base_string = '{0};ID={1}'.format(svtype, bedpe_name)
        if allele_frequency is not None:
            base_string += ';AF={0}'.format(allele_frequency)
        if strands is not None:
            base_string += ';STR={0}'.format(''.join(strands))
        return base_string

    def distant_coordinates(self, strand, start, stop):
        '''
        Transform coordinates if SV is too large
        '''
        if strand == '+':
            stop += self.coordinate_buffer
        else:
            start -= self.coordinate_buffer
        return (start, stop)

    def distant_block_sizes(self, strand, start, stop):
        '''
        Return tuple of chunk sizes
        '''
        rv = (stop - start, 1)
        if strand == '+':
            return rv
        else:
            return rv[::-1] #reverse

    def distant_block_starts(self, strand, start, stop):
        '''
        Return tuple of chunk sizes
        '''
        second_start = self.coordinate_buffer
        if strand == '+':
            second_start += (stop - start) - 1
        return (0, second_start)

    @staticmethod
    def create_line(chrom, start, end, name, score, strand, color, size_tuple=None, start_tuple=None):
        '''
        Create a blockedBed line
        '''
        fields = [
            chrom, 
            start, 
            end, 
            name, 
            score, 
            strand, 
            start, 
            end, 
            color
            ]
        if size_tuple is not None and start_tuple is not None:
            fields += [
                    '2',
                    ','.join(map(str, size_tuple)),
                    ','.join(map(str,start_tuple))
            ]
        return '\t'.join(map(str,fields))

    def convert(self, bedpe):
        '''
        Convert Bedpe line to a Bed12 line
        '''
        span = abs(bedpe.e2 - bedpe.s1)
        color = self.get_color(bedpe.svtype, span)

        output_lines = list()
        if (bedpe.svtype != 'BND'):
            name = self.bed12_name(bedpe.svtype, bedpe.name, bedpe.af)
            if span <= self.max_dist:
                output_lines.append(self.create_line(
                    bedpe.c1, 
                    bedpe.s1, 
                    bedpe.e2, 
                    name, 
                    bedpe.score, 
                    '+',
                    color, 
                    (bedpe.e1 - bedpe.s1, bedpe.e2 - bedpe.s2),
                    (0, bedpe.s2 - bedpe.s1)))
            else:
                s1, e1 = self.distant_coordinates(bedpe.o1, bedpe.s1, bedpe.e1)
                size_tuple1 = self.distant_block_sizes(bedpe.o1, bedpe.s1, bedpe.e1)
                start_tuple1 = self.distant_block_starts(bedpe.o1, bedpe.s1, bedpe.e1)
                output_lines.append(self.create_line(
                    bedpe.c1, 
                    s1, 
                    e1, 
                    name, 
                    bedpe.score, 
                    bedpe.o1, 
                    color, 
                    size_tuple1,
                    start_tuple1))
                s2, e2 = self.distant_coordinates(bedpe.o2, bedpe.s2, bedpe.e2)
                size_tuple2 = self.distant_block_sizes(bedpe.o2, bedpe.s2, bedpe.e2)
                start_tuple2 = self.distant_block_starts(bedpe.o2, bedpe.s2, bedpe.e2)
                output_lines.append(self.create_line(
                    bedpe.c2,
                    s2,
                    e2,
                    name,
                    bedpe.score,
                    bedpe.o2,
                    color,
                    size_tuple2,
                    start_tuple2))
        else:
            name = self.bed12_name(bedpe.svtype, bedpe.name, bedpe.af, (bedpe.o1, bedpe.o2))
            output_lines.append(self.create_line(
                bedpe.c1,
                bedpe.s1,
                bedpe.e1,
                name, 
                bedpe.score,
                bedpe.o1,
                color))
            output_lines.append(self.create_line(
                bedpe.c2,
                bedpe.s2,
                bedpe.e2,
                name, 
                bedpe.score,
                bedpe.o2,
                color))
        return output_lines

def processBEDPE(bedpe_stream, name, dist, output_handle=sys.stdout):
    #Process the BEDPE file and convert each entry to SAM.
    converter = BedpetoBlockedBedConverter(name, dist)
    output_handle.write(converter.track_name())
    for line in bedpe_stream:
        # ignore header
        if line[0] == "#":
            continue
        lineList = line.rstrip().split('\t')
        if lineList:
            bedpe = Bedpe(lineList)
            output_handle.write('\n'.join(converter.convert(bedpe)) + '\n')

def description():
    return 'convert a BEDPE file to BED12 format for viewing in IGV or the UCSC browser'

def epilog():
    return 'The input BEDPE file may be gzipped. If the input file is omitted then input is read from stdin. Output is written to stdout.' 

def add_arguments_to_parser(parser):
    parser.add_argument('-b', '--bedpe', metavar='<BEDPE>', default=None, help='BEDPE input file')
    parser.add_argument('-n', '--name', metavar='<STRING>', default='BEDPE', help="The name of the track. Default is 'BEDPE'")
    parser.add_argument('-d', '--maxdist', metavar='<INT>', dest='dist', default=1000000, type=int, help='The minimum distance for drawing intrachromosomal features as if they are interchromosomal (i.e., without a line spanning the two footprints). Default is 1Mb.')
    parser.set_defaults(entry_point=run_from_args)

def command_parser():
    parser = argparse.ArgumentParser(description=description(), epilog=epilog())
    add_arguments_to_parser(parser)
    return parser

def run_from_args(args):
    with su.InputStream(args.bedpe) as stream:
        processBEDPE(stream, args.name, args.dist)

if __name__ == "__main__":
    parser = command_parser()
    args = parser.parse_args()
    sys.exit(args.entry_point(args))



import re
import time
from six.moves import xrange

class Vcf(object):
    '''
    This class tracks meta information about the samples and tags in the VCF file
    It largely corresponds to the information stored in the VCF header.
    '''
    def __init__(self):
        '''
        Initialize a new object

        The GT format field is added as the Genotype class always adds this field
        '''
        self.file_format = 'VCFv4.2'
        self.other_meta = []
        self.sample_list = []
        self.sample_indices = dict()
        self.info_list = []
        self.format_list = []
        self.filter_list = []
        self.alt_list = []
        self.add_format('GT', 1, 'String', 'Genotype')

    def parse_meta(self, line):
        '''
        Parse out a meta information line from the VCF header into its constituents.
        Note that this doesn't handle comment-style lines e.g. ##BLAHBLAH
        '''
        a = line[line.find('<')+1:line.rfind('>')]
        r = re.compile(r'(?:[^,\"]|\"[^\"]*\")+')
        return r.findall(a)

    def add_header(self, header):
        '''
        Populate the meta information by reading the header
        '''
        # XXX We could probably simplify the below by making a dict of our supported keys
        # and have the values by method/function references for the action
        for line in header:
            split_header = line.split('=')
            if split_header[0] == '##fileformat':
                self.file_format = line.rstrip().split('=')[1]
            elif split_header[0] == '##fileDate':
                self.other_meta.append('##fileDate=' + time.strftime('%Y%m%d'))
            elif split_header[0] == '##INFO':
                self.add_info(*[b.split('=')[1] for b in self.parse_meta(line)])
            elif split_header[0] == '##ALT':
                self.add_alt(*[b.split('=')[1] for b in self.parse_meta(line)])
            elif split_header[0] == '##FORMAT':
                self.add_format(*[b.split('=')[1] for b in self.parse_meta(line)])
            elif split_header[0] == '##FILTER':
                self.add_filter(*[b.split('=')[1] for b in self.parse_meta(line)])
            elif split_header[0].startswith('##'):
                self.other_meta.append(line.rstrip())
            elif line[0] == '#' and line[1] != '#':
                self.sample_list = line.rstrip().split('\t')[9:]
                for i in xrange(0, len(self.sample_list)):
                    self.sample_indices[self.sample_list[i]] = i + 9

    # return the VCF header
    def get_header(self, include_samples=True):
        '''
        Generate the header string. Last line doesn't contain a newline.
        '''
        header_meta_array = ['##fileformat=' + self.file_format] + \
                        self.other_meta + \
                        [i.hstring for i in self.info_list] + \
                        [a.hstring for a in self.alt_list] + \
                        [f.hstring for f in self.filter_list] + \
                        [f.hstring for f in self.format_list]
        header_array = ['#CHROM',
                        'POS',
                        'ID',
                        'REF',
                        'ALT',
                        'QUAL',
                        'FILTER',
                        'INFO']
        if include_samples:
            header_array += ['FORMAT'] + self.sample_list
        return '\n'.join(header_meta_array + ['\t'.join(header_array)])

    def add_info(self, id, number, type, desc):
        '''
        Add new meta information about an INFO tag/field
        '''
        if id not in [i.id for i in self.info_list]:
            inf = self.Info(id, number, type, desc)
            self.info_list.append(inf)

    def add_info_after(self, insert_id, id, number, type, desc):
        '''
        Add new meta information about an INFO tag/field after another field

        This matters as the order of tags is used to determine the order of their output
        during string construction.
        '''
        for i in range(0, len(self.info_list)):
            if insert_id == self.info_list[i].id and (id not in [j.id for j in self.info_list]):
                inf = self.Info(id, number, type, desc)
                self.info_list.insert(i + 1, inf)
                return

    def add_alt(self, id, desc):
        '''
        Add a new symbolic ALTernate allele meta information line
        '''
        if id not in [a.id for a in self.alt_list]:
            alt = self.Alt(id, desc)
            self.alt_list.append(alt)

    def add_format(self, id, number, type, desc):
        '''
        Add meta info about a FORMAT field/tag
        '''
        if id not in [f.id for f in self.format_list]:
            fmt = self.Format(id, number, type, desc)
            self.format_list.append(fmt)

    def add_filter(self, id, desc):
        '''
        Add meta info about a FILTER
        '''
        if id not in [f.id for f in self.filter_list]:
            flt = self.Filter(id, desc)
            self.filter_list.append(flt)

    def add_sample(self, name):
        '''
        Add a sample to our VCF. We store the index.
        '''
        self.sample_list.append(name)
        # XXX Probably slow. Optimize if we start adding lots of samples ever
        self.sample_indices[name] = self.sample_list.index(name) + 9

    def sample_to_col(self, sample):
        '''
        Get the VCF column index of a sample. This is zero-based, like python arrays
        '''
        return self.sample_indices[sample]


    class Info(object):
        '''
        Class for storing information about INFO fields
        '''
        def __init__(self, id, number, type, desc):
            '''
            Initialize values
            '''
            self.id = str(id)
            self.number = str(number)
            self.type = str(type)
            self.desc = str(desc)
            # strip the double quotes around the string if present
            if self.desc.startswith('"') and self.desc.endswith('"'):
                self.desc = self.desc[1:-1]
            self.hstring = '##INFO=<ID=' + self.id + ',Number=' + self.number + ',Type=' + self.type + ',Description=\"' + self.desc + '\">'
        def __eq__(self, other):
            '''
            Compare two INFO field meta lines
            '''
            return self.hstring == other.hstring

    class Alt(object):
        '''
        Class for storing information about symbolic
        ALTernate alleles
        '''
        def __init__(self, id, desc):
            '''
            Initialize values
            '''
            self.id = str(id)
            self.desc = str(desc)
            # strip the double quotes around the string if present
            if self.desc.startswith('"') and self.desc.endswith('"'):
                self.desc = self.desc[1:-1]
            self.hstring = '##ALT=<ID=' + self.id + ',Description=\"' + self.desc + '\">'

        def __eq__(self, other):
            '''
            Compare two ALT field meta lines
            '''
            return self.hstring == other.hstring

    class Format(object):
        '''
        Class for storing information about
        Format fields
        '''
        def __init__(self, id, number, type, desc):
            '''
            Initialize values
            '''
            self.id = str(id)
            self.number = str(number)
            self.type = str(type)
            self.desc = str(desc)
            # strip the double quotes around the string if present
            if self.desc.startswith('"') and self.desc.endswith('"'):
                self.desc = self.desc[1:-1]
            self.hstring = '##FORMAT=<ID=' + self.id + ',Number=' + self.number + ',Type=' + self.type + ',Description=\"' + self.desc + '\">'
        def __eq__(self, other):
            '''
            Compare two FORMAT meta lines
            '''
            return self.hstring == other.hstring

    class Filter(object):
        '''
        Class for storing info about FILTER fields
        '''
        def __init__(self, id, desc):
            '''
            Initialize values
            '''
            self.id = str(id)
            self.desc = str(desc)
            # strip the double quotes around the string if present
            if self.desc.startswith('"') and self.desc.endswith('"'):
                self.desc = self.desc[1:-1]
            self.hstring = '##FILTER=<ID=' + self.id + ',Description=\"' + self.desc + '\">'

        def __eq__(self, other):
            '''
            Compare two lines
            '''
            return self.hstring == other.hstring
